@@ -4,7 +4,7 @@ import random
 from collections import defaultdict
 from functools import lru_cache
 
-from criteria_local.criteria import CriteriaLocal
+from criteria_local.criteria_profile import CriteriaProfile
 from database_mysql_local.generic_crud import GenericCRUD
 from language_remote.lang_code import LangCode
 from logger_local.MetaLogger import MetaLogger
@@ -49,13 +49,15 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
 
         self.profile_local = ProfilesLocal(is_test_data=is_test_data)
         self.message_template = MessageTemplates(is_test_data=is_test_data)
-        self.criteria_local = CriteriaLocal(is_test_data=is_test_data)
+        self.criteria_profile = CriteriaProfile(is_test_data=is_test_data)
         self.user_context = UserContext()
         self.queue = None
 
         self.set_compound_message_after_text_template()
 
+    # TODO If it is being used only for outgoing messaeges, shall we rename the method to get_outgoing_messages_queue()
     def get_queue(self):
+        # TODO Shall we change self.queue to self.outgoing_message_queue so it will be easier to search the code in the IDE
         if not self.queue:
             self.queue = DatabaseQueue(schema_name="message", table_name="message_table",
                                        view_name="message_view", queue_item_id_column_name="message_id",
@@ -112,7 +114,8 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
     #         criteria_set_ids_per_message_template_id[row["message_template_id"]].append(row["criteria_set_id"])
     #     return criteria_set_ids_per_message_template_id
 
-    def get_recipients_by_criteria_set_ids(self, criteria_set_ids: list[int]) -> list[Recipient]:
+    @lru_cache
+    def get_recipients_by_criteria_set_ids(self, criteria_set_ids: tuple[int, ...]) -> list[Recipient]:
         query = f"""
             SELECT phone_profile.full_number_normalized           AS phone_number,
                    phone_profile.`profile.main_email_address`     AS email_address,
@@ -125,7 +128,7 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
         """
         all_profile_ids = []
         if criteria_set_ids:  # otherwise get all profiles
-            profile_ids_per_criteria_set_id = self.criteria_local.get_profile_ids_per_criteria_set_id(criteria_set_ids)
+            profile_ids_per_criteria_set_id = self.criteria_profile.get_profile_ids_per_criteria_set_id(criteria_set_ids)
             all_profile_ids = list(set([profile_id for profile_ids in profile_ids_per_criteria_set_id.values()
                                         for profile_id in profile_ids]))
             if not all_profile_ids:
@@ -171,7 +174,7 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
 
         if not self._recipients and campaign_id:
             # Get campaign recipients:
-            criteria_set_ids = self.criteria_local.get_criteria_set_ids_list_by_campaign_id(campaign_id=campaign_id)
+            criteria_set_ids = self.criteria_profile.get_criteria_set_ids_list_by_campaign_id(campaign_id=campaign_id)
             self._recipients = self.get_recipients_by_criteria_set_ids(criteria_set_ids)
 
         compound_message = {"json_version": JSON_VERSION, "data": {}}
@@ -230,7 +233,6 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
         self.logger.debug(object=locals())
 
     def _split_and_push_compound_message(self, compound_message: dict, is_require_moderator: bool) -> list:
-        message_ids = []
         all_profile_ids = sorted({
             profile["profile_id"]
             for channel_key, channel_value in compound_message["data"].items()
@@ -239,6 +241,11 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
             for text_block in message_template["MessageTemplateTextBlocks"]
             for profile in text_block["Profiles"]})
 
+        if len(all_profile_ids) <= RECIPIENTS_LIMIT_PER_COMPOUND_MESSAGE:  # for performance reasons
+            message_ids = [self._push_compound_message(compound_message, is_require_moderator)]
+            return message_ids
+
+        message_ids = []
         for i in range(0, len(all_profile_ids), RECIPIENTS_LIMIT_PER_COMPOUND_MESSAGE):
             profile_ids = all_profile_ids[i:i + RECIPIENTS_LIMIT_PER_COMPOUND_MESSAGE]
             self.logger.info(f"Processing profiles: {profile_ids}")
@@ -274,16 +281,21 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
                 if mini_channel_value["Page"]:
                     mini_compound_message['data'][channel_key] = mini_channel_value
 
-            data_dict = {
-                "compound_message_json": json.dumps(mini_compound_message),
-                "compound_message_json_version": JSON_VERSION,
-                "is_require_moderator": is_require_moderator}
-            data_dict.update(self.save_additional_parameters)
-            message_id = self.get_queue().push(data_dict)
-
+            message_id = self._push_compound_message(mini_compound_message, is_require_moderator)
             message_ids.append(message_id)
         return message_ids
 
+    def _push_compound_message(self, compound_message: dict, is_require_moderator: bool) -> int:
+        # TODO data_dict -> outgoing_message_dict
+        # TODO Can we also add message_template_id to outgoing_message_dict
+        data_dict = {
+            "compound_message_json": json.dumps(compound_message),
+            "compound_message_json_version": JSON_VERSION,
+            "is_require_moderator": is_require_moderator}
+        data_dict.update(self.save_additional_parameters)
+        message_id = self.get_queue().push(data_dict)
+        return message_id
+
     @staticmethod
     def _is_processed(*, profile_id: int, message_template_text_block_id: int, processed_text_blocks: list) -> bool:
         is_processed = any(processed_text_block["profile_id"] == profile_id and
@@ -298,8 +310,8 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
                             content_per_language: dict, possible_answers_per_question_id: dict,
                             processed_text_blocks: list) -> list[dict]:
         if text_block.get("criteria_set_id"):
-            potentials_recipients = self.criteria_local.get_profile_ids_per_criteria_set_id(
-                criteria_set_ids=[text_block["criteria_set_id"]]).get(text_block["criteria_set_id"], [])
+            potentials_recipients = self.criteria_profile.get_profile_ids_per_criteria_set_id(
+                criteria_set_ids=(text_block["criteria_set_id"], )).get(text_block["criteria_set_id"], [])
         else:
             potentials_recipients = [recipient.get_profile_id() for recipient in recipients]
 
