@@ -84,8 +84,17 @@ class MessageLocal(Item, CompoundMessage, metaclass=MetaLogger, object=object_me
         #     body = self.message_template.get_message_template_textblock_and_attributes(
         #         message_template_id=body, destination_profile_id=recipient.get_profile_id())
 
+    def insert_message_data(self, message_data: dict) -> int:
+        """
+        Inserts message data into the database.
+        """
+        self._validate_args(message_data)
+        message_id = self.insert(data_dict=message_data)
+        return message_id
+
     def get_id(self) -> int or None:
-        return self.message_id
+        message_id = self.message_ids[0] if self.message_ids else None
+        return message_id
 
     def get_message_channel_by_recipient(self, recipient: Recipient) -> MessageChannel:
         # TODO: return msg_type (sms, email, whatsapp) based on hours, provider availability, msg length, etc.
@@ -96,14 +105,17 @@ class MessageLocal(Item, CompoundMessage, metaclass=MetaLogger, object=object_me
         4. time of the day
         5. preferences of the recipient
         6. attachments type and size 7. cost of sending the message"""
-        for message_channel in MessageChannel:
-            if recipient.get_email_address() is not None:
-                return MessageChannel.EMAIL
-            elif len(self.get_body_text_after_template_processing(
-                    recipient=recipient, message_channel=message_channel)) < SMS_MESSAGE_LENGTH:
-                return MessageChannel.SMS
-            else:
-                return MessageChannel.WHATSAPP
+        if recipient.get_email_address() is not None:
+            message_channel = MessageChannel.EMAIL
+        else:
+            for _message_channel in MessageChannel:
+                if len(self.get_body_text_after_template_processing(
+                        recipient=recipient, message_channel=_message_channel)) < SMS_MESSAGE_LENGTH:
+                    message_channel = MessageChannel.SMS
+                    return message_channel
+            message_channel = MessageChannel.WHATSAPP
+        return message_channel
+        
 
     # TODO Let's create logger_table SQL to verify the decision taken by this method (like ...)
     def get_message_channel(self, recipient: Recipient) -> MessageChannel:
@@ -115,18 +127,25 @@ class MessageLocal(Item, CompoundMessage, metaclass=MetaLogger, object=object_me
 
     # TODO Let's create logger_table SQL to verify the decision taken by this method (like ...)
     @staticmethod
+    # TOOD get_message_channel_provider_id
     def get_message_provider_id(*, message_channel: MessageChannel, recipient: Recipient) -> int:
         """return message provider"""
         # TODO: implement the logic for the provider_id (remove +972, etc.)
+        # TODO message_channel_provider_id
+        # TODO Sending SMS only in Israel, the rest will be subjected to moderation
+        # TODO Can we use Israel country_id instead of 972 to make it more generic and readable?
+        # TODO Object Oriented wise, I would expect this logic to be in https://github.com/circles-zone/sms-message-local-python-package
         if message_channel == MessageChannel.SMS and recipient.get_phone_number_full_normalized().startswith("972"):
             provider_id = AWS_SNS_SMS_MESSAGE_PROVIDER_ID
         elif message_channel == MessageChannel.EMAIL:
             provider_id = AWS_SES_EMAIL_MESSAGE_PROVIDER_ID
         elif message_channel == MessageChannel.WHATSAPP:
             # TODO: or vonage
+            # TODO Can we use WHATSAPP_SELENIUM_MESSAGE_CHANNEL_PROVIDER_ID?
             provider_id = INFORU_MESSAGE_PROVIDER_ID
         else:
-            raise Exception("Can't determine the Message Provider for message_channel=" + str(message_channel))
+            # TODO Can we include the message_id?
+            raise Exception("Can't determine the Message Channel Provider for message_channel=" + str(message_channel))
 
         return provider_id
 
@@ -263,10 +282,24 @@ class MessageLocal(Item, CompoundMessage, metaclass=MetaLogger, object=object_me
         self.logger.info("http_status_code: " + str(http_status_code))
         return http_status_code == HTTPStatus.OK.value
 
+    # TOOO Do we want to send only to one channel or multi-channels of the recipients? - Please add support to campaign_criteria_set_table.number_of_channels_per_recipient
     def send(self, *, body: str = None, compound_message_dict: dict = None,
              recipients: List[Recipient] = None, cc: List[Recipient] = None, bcc: List[Recipient] = None,
              scheduled_timestamp_start: datetime = None,
              scheduled_timestamp_end: datetime = None, **kwargs) -> list[int]:
+        '''This method supports multiple alternatives to send messages
+            1. body only
+            2. compound_message created by CompoundMessage class
+            
+            1. with/without recipients/to
+            2. with/without cc
+            3. with/without bcc
+            
+            1. Without schedule
+            2. With schedule_timestamp_start
+            3. With schedule_timestamp_end
+            4. With both start and end
+        '''
         pass  # this is an abstract method, but we don't want to make this class abstract
 
     def after_send_attempt(self, *, sender_profile_id: int = None, outgoing_body: json = None,
